mod api;

use actix_multipart::form::tempfile::TempFileConfig;
use actix_web::{middleware, App, HttpServer};
use api::camp_form::camp_form;
use api::church_form::church_form;
use api::health_form::health_form;
use api::local_trip_form::local_form;
use api::mt_form::mt_form;
use api::parent_form::parent_form;
use api::teacher_form::teacher_form;

#[actix_web::main]
async fn main() -> std::io::Result<()> {
    env_logger::init_from_env(env_logger::Env::new().default_filter_or("info"));

    log::info!("creating temporary upload directory");
    std::fs::create_dir_all("./tmp")?;

    log::info!("starting HTTP server at http://localhost:4242");

    HttpServer::new(|| {
        App::new()
            .wrap(middleware::Logger::default())
            .app_data(TempFileConfig::default().directory("./tmp"))
            .service(mt_form)
            .service(health_form)
            .service(parent_form)
            .service(teacher_form)
            .service(church_form)
            .service(local_form)
            .service(camp_form)
    })
    .bind(("127.0.0.1", 4242))?
    .workers(2)
    .run()
    .await
}
